import org.gicentre.processing.utils.TextPopup;
import java.util.Random;

// Sketch to show how a text popup window can be added to a sketch
// Version 1.0, 9th October, 2009.
// Author Jo Wood.

// ------------------ Object variables --------------------

private TextPopup textPopup;

// ------------------- Initialisation ---------------------

/** Sets up the sketch by initialising the text popup.
 */
void setup()
{
  size(800,600);
  smooth();
  frame.setResizable(true);
  strokeWeight(2);  
  
  // The text popup needs to know the sketch (this) in which to draw.
  // The last two parameters define the width and height of the border around 
  // the popup relative to the size of this sketch.
  textPopup = new TextPopup(this,50,80);
  textPopup.setTextSize(24);
  textPopup.setInternalMargin(18,9);
  textPopup.addText("An example title",32);
  textPopup.addText("");    // Blank line
  textPopup.addText("H - Toggle help screen on and off");
  textPopup.addText("Up/down - Increase/decrease size of internal margin");
  textPopup.addText("Left/right - Decrease/increase size of external margin");
  textPopup.addText("<Esc> - Quit the sketch");
  textPopup.addText("\n\n"); // Two blank lines
  textPopup.addText("You can change the size of the fonts displayed in this window. If the text "+
                    "is too large to fit on a single line, it will get wrapped onto the next line, "+
                    "so a large block of text can be displayed easily in this window.", 16);
}

// ------------------ Processing draw --------------------

/** Draws a simple sketch with the option of displaying the colour picker.
 */
void draw()
{
  background(255);
  
  // Draw some random stuff to represent a sketch.
  stroke(180,0,0);
  Random rand = new Random(5432);
  
  for (int i=0; i<100; i++)
  {
    line(rand.nextFloat()*width,rand.nextFloat()*height, 
         rand.nextFloat()*width,rand.nextFloat()*height);
  }
  
  // Add this to display the colour picker.
  textPopup.draw();
}

// ------------------ Keyboard handlinng --------------------

/** Turns the colour picker on or off with the 'c' key.
 */
void keyPressed()
{ 
  if ((key == 'h') || (key == 'H'))
  {
    textPopup.setIsActive(!textPopup.getIsActive());
  }
  
  if (textPopup.getIsActive())
  {
    if (key== CODED)
    {
      int margin = textPopup.getInternalMargin().width;
      int border = textPopup.getExternalMargin().width;
    
      if (keyCode == UP)
      {
        margin++;
      }
      else if (keyCode == DOWN)
      {
        margin--;
      }
      textPopup.setInternalMargin(margin,margin/2);  
    
      if (keyCode == LEFT)
      {
        border-=4;
      }
      else if (keyCode == RIGHT)
      {
        border+=4;
      }
      textPopup.setExternalMargin(border,border);  
    }
  }
}

